/** The AST node for types created by class declarations. */

package AST;

import SymbolTable.*;
import IL.*;
import Helpers.*;

public class ClassType extends Type {
    // Instance variables

    // AST subtrees:
    public String class_name;

    // The resolved type, set after typechecking
    protected ResolvedClassType resolved_type;


    // Constructor
    public ClassType(String cn, int ln) {
	super(ln);
	class_name = cn;
    }


    // AST printing
    public String toString(String indent) {
	return class_name;
    }


    // Typechecking operations

    /** Check that this type is a legal type, and return the
     * corresponding resolved type to use during typechecking. */
    public ResolvedType resolve(SymbolTable st)
		throws TypecheckCompilerException {
	// compute the resolved type
	resolved_type = ResolvedType.classType(class_name, st, line_number);
	// return it
	return resolved_type;
    }

    /** Return the resolved version of this type, after typechecking. */
    public ResolvedType resolvedType() {
	if (resolved_type == null) {
	    throw new InternalCompilerError(
		    "getting the resolved type before typechecking");
	}
	return resolved_type;
    }


    // Lowering operations

    /** Return the lowered version of this type */
    public ILType lower() {
	return resolved_type.lower();
    }
}
