/** The abstract superclass of all AST nodes for types. */

package AST;

import SymbolTable.*;
import IL.*;
import Helpers.*;

public abstract class Type extends ASTNode {
    // Constructor
    public Type(int ln) {
	super(ln);
    }


    // Typechecking operations

    /** Check that this type is a legal type, and return the
     * corresponding resolved type to use during typechecking.
     * implemented by subclasses. */
    public abstract ResolvedType resolve(SymbolTable st)
	throws TypecheckCompilerException;

    /** Return the resolved version of this type, after typechecking. */
    public abstract ResolvedType resolvedType();


    // Lowering operations

    /** Return the lowered version of this type */
    public abstract ILType lower();
}
