/** The AST node for a local var decl statement. */

package AST;

import SymbolTable.*;
import Environment.*;
import IL.*;
import Helpers.*;

public class VarDeclStmt extends Stmt {
    // Instance variables

    // AST subtrees:
    public VarDecl decl;


    // Constructor
    public VarDeclStmt(VarDecl d, int ln) {
	super(ln);
	decl = d;
    }


    // AST printing
    public String toString(String indent) {
	return decl.toString(indent) + ";";
    }


    // Typechecking operations

    /** Typecheck this statement in the given enclosing context */
    public void typecheck(CodeSymbolTable st)
		throws TypecheckCompilerException {
	// declare the new local variable, checking for duplicates
	st.declareLocalVariable(decl.type.resolve(st), decl.name, line_number);
    }


    // Evaluation operations

    /** Evaluate this statement in the given environment */
    public void evaluate(CodeEnvironment env) throws EvalCompilerException {
	// declare the variable in the current environment
	env.declareLocalVariable(decl.name, line_number);
    }


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language function with lowered code
     * for this statement */
    public void lower(ILFunDecl il_fun_decl) {
	// declare this as a new local variable in the IL function
	il_fun_decl.declareLocal(decl.type.lower(), decl.name);
	// (maybe someday we'd want to insert dummy initializations,
	// to ensure that uninitialized variables aren't read)
    }
}
