/** The AST node for a variable reference expression. */

package AST;

import SymbolTable.*;
import Environment.*;
import IL.*;
import Helpers.*;

public class VarExpr extends Expr {
    // Instance variables

    // AST subtrees:
    public String name;

    // The information about the variable, set after typechecking
    protected VarInterface var_interface;


    // Constructor
    public VarExpr(String n, int ln) {
	super(ln);
	name = n;
    }


    // AST printing
    public String toString(String indent) {
	return name;
    }


    // Typechecking operations

    /** Typecheck this expression in the given enclosing context,
     * returning the type of the result */
    protected ResolvedType typecheckHelper(CodeSymbolTable st)
		throws TypecheckCompilerException {
	// look up and remember the information about the variable
	var_interface = st.lookupVariable(name, line_number);
	
	// return the type of the variable
	return var_interface.getType();
    }


    // Evaluation operations

    /** Evaluate this expression in the given environment, returning
     * the value of the result */
    public Value evaluate(CodeEnvironment env) throws EvalCompilerException {
	// look up the variable's current value
	return var_interface.lookupVariable(env, line_number);
    }


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language function with code to
     * evaluate this expression, returning an IL expression for
     * computing the result */
    public ILExpr lower(ILFunDecl il_fun_decl) {
	// generate the variable read, based on the kind of variable
	// (class or local) we're reading
	return var_interface.generateRead(il_fun_decl);
    }
}
