/** The abstract superclass for environments, which store information
 * during evaluation about a particular scope */

package Environment;

import Helpers.*;

public abstract class Environment {
    // Constructor
    public Environment() { }


    // Accessors

    /** Return the environment for the class named 'class_name'. */
    public abstract ClassEnvironment lookupClass(String name, int line_number);

    /** assign to an instance variable of the given receiver object */
    public abstract void assignInstanceVariable(Value receiver, String name,
						Value value, int line_number);

    /** look up the current contents of an instance variable of the
     * given receiver object */
    public abstract Value lookupInstanceVariable(Value receiver, String name,
						 int line_number);

    /** assign to a local variable */
    public abstract void assignLocalVariable(String name, Value value, 
					     int line_number);

    /** look up the current contents of a local variable, checking to
     * make sure that we don't access the variable before it's been
     * assigned to */
    public abstract Value lookupLocalVariable(String name, int line_number)
	throws EvalCompilerException;


    // Tracing operations

    /** whether to print information about calls and returns */
    public abstract boolean printCalls();

    /** return the number of callers of this method environment (0 =
     * the main class's main method) */
    public int caller_depth() {
	// this should only end up being called on a CodeEnvironment
	throw new InternalCompilerError("shouldn't be here");
    }
}
