/** The class for an environment for a scope that's lexically nested
 * within some other scope. */

package Environment;

import Helpers.*;

public abstract class NestedEnvironment extends Environment {
    /** the environment for the lexically enclosing scope */
    protected Environment enclosing_env;

    public NestedEnvironment(Environment e) { 
	enclosing_env = e;
    }


    // Accessors

    /** Return the environment for the class named 'class_name'. */
    public ClassEnvironment lookupClass(String name, int line_number) {
	// look in the lexically enclosing scope's environment
	return enclosing_env.lookupClass(name, line_number);
    }

    /** assign to an instance variable of the given receiver object */
    public void assignInstanceVariable(Value receiver, String name,
				       Value value, int line_number) {
	// look in the lexically enclosing scope's environment
	enclosing_env.assignInstanceVariable(receiver, name, value,
					     line_number);
    }

    /** look up the current contents of an instance variable of the
     * given receiver object */
    public Value lookupInstanceVariable(Value receiver, String name,
					int line_number) {
	// look in the lexically enclosing scope's environment
	return enclosing_env.lookupInstanceVariable(receiver, name,
						    line_number);
    }


    // Tracing operations

    /** whether to print information about calls and returns */
    public boolean printCalls() {
	// stored in the outermost environment
	return enclosing_env.printCalls();
    }

    /** return the number of callers of this method environment (0 =
     * the main class's main method) */
    public int caller_depth() {
	// skip out to the enclosing method environment
	return enclosing_env.caller_depth();
    }
}
