/** The class for the null value. */

package Environment;

import Helpers.*;

public class NullValue extends Value {
    // the single null value instance
    protected static NullValue nullValue = new NullValue();

    // Constructor (protected so that clients can't call it directly)
    protected NullValue() { }

    // Public static operation to get the single null value
    // instance (this is what clients should call)
    public static NullValue nullValue() {
	return nullValue;
    }


    // Printing operations
    public String toString() { 
	return "null";
    }


    // Evaluation operations

    /** If the value is a class instance, look up the method with the
     * given name in the receiver instance, throwing an evaluation
     * error if the receiver is null; otherwise report an internal
     * error.  return the method declaration that is invoked. */
    public MethodDeclAndEnv lookupMethod(String name, int line_number)
		throws EvalCompilerException {
	throw new EvalCompilerException("dereferencing null", line_number);
    }
}
