/** This is the abstract superclass for all values computed during
 * evaluation. */

package Environment;

import SymbolTable.*;
import Helpers.*;
import java.util.*;

public abstract class Value {
    // Printing operations
    public abstract String toString();


    // Evaluation operations

    /** Return whether the value is the special uninitialized value */
    public boolean isUninitialized() {
	// by default, it's not
	return false;
    }

    /** If the value is an int, return its value, otherwise report
     * an internal error */
    public int getIntValue(int line_number) {
	throw new InternalCompilerError("expecting an int", line_number);
    }

    /** If the value is a boolean, return its value, otherwise report
     * an internal error */
    public boolean getBooleanValue(int line_number) {
	throw new InternalCompilerError("expecting a boolean", line_number);
    }

    /** If the value is a class instance, look up the method with the
     * given name in the receiver instance, throwing an evaluation
     * error if the receiver is null; otherwise report an internal
     * error.  return the method declaration that is invoked. */
    public MethodDeclAndEnv lookupMethod(String name, int line_number)
		throws EvalCompilerException {
	throw new InternalCompilerError("expecting a class instance or null",
					line_number);
    }

    /** If the value is a class instance, return the contents of one
     * of its instance variables, otherwise report an internal
     * error */
    public Value lookupInstanceVariable(String name, int line_number) {
	throw new InternalCompilerError("expecting a class instance",
					line_number);
    }

    /** If the value is a class instance, assign to one of its
     * instance variables, otherwise report an internal error */
    public void assignInstanceVariable(String name, Value value,
				       int line_number) {
	throw new InternalCompilerError("expecting a class instance",
					line_number);
    }
}
