/** This is the class of IL (intermediate language) expressions that
 * are memory allocations.  The allocated memory is not initialized. */

package IL;

import Target.*;
import java.io.*;

public class ILAllocateExpr extends ILExpr {
    // Instance variable
    protected ILType type; // the type of the thing allocated; the
			   // result is a pointer to a value of this
			   // type


    // Constructor
    public ILAllocateExpr(ILType t) {
	type = t;
    }


    // Printing
    public String toString() {
	return "new " + type.toString();
    }


    // Target code generation operations

    /** generate target code for this expression, given the set of
     * scratch registers available & allocated. return the location
     * where the result is. */
    public Location codegen(Target target) throws IOException {
	return target.emitAllocate(type);
    }
}
