/** The class for IL array types.  Only a single instance should ever
 * be created for each element type. */

package IL;

import Target.*;
import java.util.*;

public class ILArrayType extends ILType {
    // Instance variable

    // the array's element type
    protected ILType elem_type;

    // protected static variable, caching the unique representatives of
    // each distinct array type
    protected static Map/*<ILType,ILArrayType>*/ arrayTypes = 
	new HashMap();

    // Constructor (protected, so clients can't call it directly)
    protected ILArrayType(ILType et) {
	elem_type = et;
    }

    // public static operation, which is the main one clients should
    // use to get the unique instance of a particular array type
    public static ILArrayType arrayILType(ILType elem_type) {
	Object pt = arrayTypes.get(elem_type);
	if (pt == null) {
	    // array type not already created
	    
	    // create it
	    ILArrayType rpt = new ILArrayType(elem_type);

	    // store the new resolved type into the table
	    arrayTypes.put(elem_type, rpt);

	    // return the new resolved class type
	    return rpt;
	} else {
	    // array type found
	    return (ILArrayType)pt;
	}
    }


    // Accessors

    // if an array type, then return the type of the array elements.
    // otherwise report an internal error
    public ILType getArrayElemType() {
	return elem_type;
    }


    // Printing operations

    // return a printable string representation of this type.
    public String toString() { 
	return elem_type.toString() + "[]"; 
    }


    // Target code generation operations

    /** compute the layout of this type */
    public void doLayout(Target target) {
	// lay out the element type
	elem_type.doLayout(target);
    }

    /** return the size in bytes of a value of this type on the given
     * target machine */
    public int byteSize(Target target) {
	// for an array, we return the fixed size, ignoring any
	// elements, which is always 0 (when we allocate a real array,
	// we'll add in the space required for the number of elements
	// allocated)
	return 0;
    }

    /** return the size in bytes of each array element, assuming this
     * type ends with an array type.  report an error if not. */
    public int arrayedElementByteSize(Target target) {
	return elem_type.byteSize(target);
    }

    /** return the alignment constraint in bytes of a value of this
     * type on the given target machine, i.e., at what multiple of
     * bytes must a value of this type begin on */
    public int byteAlign(Target target) {
	// the alignment constraint of an array is that of its element
	// type
	return elem_type.byteAlign(target);
    }
}
