/** This is the class of IL (intermediate language) expressions that
 * are memory allocations of records whose last field is an array; the
 * number of array elements to allocate is given as an extra argument.
 * The allocated memory is zero'd. */

package IL;

import Target.*;
import java.io.*;

public class ILArrayedAllocateExpr extends ILAllocateExpr {
    // Instance variable
    protected ILExpr length;


    // Constructor
    public ILArrayedAllocateExpr(ILType t, ILExpr l) {
	super(t);
	length = l;
    }


    // Printing
    public String toString() {
	return super.toString() + "[" + length.toString() + "]";
    }


    // Target code generation operations

    /** generate target code for this expression, given the set of
     * scratch registers available & allocated. return the location
     * where the result is. */
    public Location codegen(Target target) throws IOException {
	return target.emitAllocateArrayed(type, length);
    }
}
