/** This is the class of IL (intermediate language) expressions that
 * are references to an array field of a pointer to a record, either
 * on the left- or right-hand-sides of assignments; the index of the
 * element to be accessed is given by an extra argument. */

package IL;

import Target.*;
import java.io.*;

public class ILArrayedFieldAccessExpr extends ILFieldAccessExpr {
    // Instance variable
    protected ILExpr index;


    // Constructor
    public ILArrayedFieldAccessExpr(ILExpr rv, ILType rt, ILRecordMember rm, 
				    ILExpr i) {
	super(rv, rt, rm);
	index = i;
    }


    // Printing
    public String toString() {
	return super.toString() + "[" + index.toString() + "]";
    }


    // Target code generation operations

    /** generate target code for this expression, given the set of
     * scratch registers available & allocated. return the location
     * where the result is. */
    public Location codegen(Target target) throws IOException {
	// make sure the record has been layed out
	record_type.doLayout(target);
	// generate the code
	return target.emitArrayedFieldRead(record_ptr, record_member, index);
    }

    /** generate target code to assign the argument to this
     * left-hand-side expression, given the set of scratch registers
     * available & allocated. */
    public void codegenAssign(Location rhs_location, Target target)
		throws IOException {
	// make sure the record has been layed out
	record_type.doLayout(target);
	// generate the code
	target.emitArrayedFieldAssign(record_ptr, record_member, index,
				      rhs_location);
    }
}
