/** The class for IL double types.  Only a single instance should
 * ever be created. */

package IL;

import Target.*;

public class ILDoubleType extends ILType {
    // protected static variable, caching the unique representative of
    // this type
    protected static ILDoubleType doubleILType = new ILDoubleType();

    // Constructor (protected, so clients can't call it directly)
    protected ILDoubleType() { }

    // public static operation, which is the main one clients should
    // use to get the unique instance of this type
    public static ILDoubleType doubleILType() { return doubleILType; }


    // Accessors

    // return whether this is the double type
    public boolean isDoubleType() {
	return true;
    }


    // Printing operations

    // return a printable string representation of this type.
    public String toString() { 
	return "double"; 
    }


    // Target code generation operations

    /** compute the layout of this type */
    public void doLayout(Target target) {
	// nothing to do
    }

    /** return the size in bytes of a value of this type on the given
     * target machine */
    public int byteSize(Target target) {
	return target.doubleByteSize();
    }

    /** return the alignment constraint in bytes of a value of this
     * type on the given target machine, i.e., at what multiple of
     * bytes must a value of this type begin on */
    public int byteAlign(Target target) {
	return target.doubleByteAlign();
    }
}
