/** The class for the IL function type.  Only a single instance should
 * ever be created.  It is a placeholder type, the target of a pointer
 * that can be invoked indirectly. */

package IL;

import Target.*;
import Helpers.*;

public class ILFunType extends ILType {
    // protected static variable, caching the unique representative of
    // this type
    protected static ILFunType funILType = new ILFunType();

    // Constructor (protected, so clients can't call it directly)
    protected ILFunType() { }

    // public static operation, which is the main one clients should
    // use to get the unique instance of this type
    public static ILFunType funILType() { return funILType; }


    // Printing operations

    // return a printable string representation of this type.
    public String toString() { 
	return "<function>"; 
    }


    // Target code generation operations

    /** compute the layout of this type */
    public void doLayout(Target target) {
	// nothing to do
    }

    /** return the size in bytes of a value of this type on the given
     * target machine */
    public int byteSize(Target target) {
	// functions are never manipulated as values directly, only
	// via pointer
	throw new InternalCompilerError(
	    "shouldn't be asking for the size of a function");
    }

    /** return the alignment constraint in bytes of a value of this
     * type on the given target machine, i.e., at what multiple of
     * bytes must a value of this type begin on */
    public int byteAlign(Target target) {
	// functions are never manipulated as values directly, only
	// via pointer
	throw new InternalCompilerError(
	    "shouldn't be asking for the alignment of a function");
    }
}
