/** The class for IL int types.  Only a single instance should
 * ever be created. */

package IL;

import Target.*;

public class ILIntType extends ILType {
    // protected static variable, caching the unique representative of
    // this type
    protected static ILIntType intILType = new ILIntType();

    // Constructor (protected, so clients can't call it directly)
    protected ILIntType() { }

    // public static operation, which is the main one clients should
    // use to get the unique instance of this type
    public static ILIntType intILType() { return intILType; }


    // Accessors

    // return whether this is the int type
    public boolean isIntType() {
	return true;
    }


    // Printing operations

    // return a printable string representation of this type.
    public String toString() { 
	return "int"; 
    }


    // Target code generation operations

    /** compute the layout of this type */
    public void doLayout(Target target) {
	// nothing to do
    }

    /** return the size in bytes of a value of this type on the given
     * target machine */
    public int byteSize(Target target) {
	return target.intByteSize();
    }

    /** return the alignment constraint in bytes of a value of this
     * type on the given target machine, i.e., at what multiple of
     * bytes must a value of this type begin on */
    public int byteAlign(Target target) {
	return target.intByteAlign();
    }
}
