/** The class for IL pointer types.  Only a single instance should
 * ever be created for each target type. */

package IL;

import Target.*;
import Helpers.*;
import java.util.*;

public class ILPtrType extends ILType {
    // Instance variable

    // the target of the pointer
    protected ILType target_type;

    // protected static variable, caching the unique representatives of
    // each distinct ptr type
    protected static Map/*<ILType,ILPtrType>*/ ptrTypes = 
	new HashMap();


    // Constructor (protected, so clients can't call it directly)
    protected ILPtrType(ILType tt) {
	target_type = tt;
    }

    // public static operation, which is the main one clients should
    // use to get the unique instance of a particular pointer type
    public static ILPtrType ptrILType(ILType target_type) {
	Object pt = ptrTypes.get(target_type);
	if (pt == null) {
	    // ptr type not already created
	    
	    // create it
	    ILPtrType rpt = new ILPtrType(target_type);

	    // store the new resolved type into the table
	    ptrTypes.put(target_type, rpt);

	    // return the new resolved class type
	    return rpt;
	} else {
	    // ptr type found
	    return (ILPtrType)pt;
	}
    }


    // Accessor

    // return whether this is a pointer type
    public boolean isPtrType() {
	return true;
    }

    // if a pointer type, then return the type of the pointer target.
    // otherwise report an internal error
    public ILType getPtrTargetType() {
	return target_type;
    }


    // Printing operations

    // return a printable string representation of this type.
    public String toString() { 
	return "*" + target_type.toString();
    }

    // return a printable string representation of this type, but
    // don't show target types below pointer types, as a way to break
    // recursion and keep types from getting too big.  typically
    // called only within ILType toString() methods.
    public String shallowToString() { 
	// don't print out the target type when doing shallow printing
	return "*?";
    }


    // Target code generation operations

    /** compute the layout of this type */
    public void doLayout(Target target) {
	// lay out the target
	target_type.doLayout(target);
    }

    /** return the size in bytes of a value of this type on the given
     * target machine */
    public int byteSize(Target target) {
	return target.ptrByteSize();
    }

    /** return the alignment constraint in bytes of a value of this
     * type on the given target machine, i.e., at what multiple of
     * bytes must a value of this type begin on */
    public int byteAlign(Target target) {
	return target.ptrByteAlign();
    }
}
