/** This is the abstract superclass of local & formal variable
 * declarations in IL (intermediate language) functions */

package IL;

import Target.*;

public abstract class ILVarDecl {
    // Instance variables
    protected ILType type;
    protected String name;

    // the byte offset in the enlosing function's stack frame for this
    // variable, relative to other variables of its kind (local or
    // formal)
    protected int relativeByteOffset = -1; // -1 means not set yet


    // Constructor
    public ILVarDecl(ILType t, String n) {
	type = t;
	name = n;
    }


    // Accessors
    public ILType getType() {
	return type;
    }

    public String getName() {
	return name;
    }

    // whether this var decl is a formal
    public abstract boolean isFormal();


    // Printing
    public String toString() {
	return type.toString() + " " + name;
    }


    // Target code generation operations

    /** set the offset in the enclosing function's stack frame for this
     * variable, relative to other variables of its kind (local or
     * formal) */
    public void setRelativeByteOffset(int offset) {
	relativeByteOffset = offset;
    }

    /** return the offset in the enclosing function's stack frame for this
     * variable, relative to the frame pointer */
    public int getByteOffset(Target target) {
	return relativeByteOffset + getBaseOffset(target);
    }

    /** return the offset in the enclosing function's stack frame for
     * this variable, relative to the frame pointer, of variables of
     * this kind (local or formal) */
    protected abstract int getBaseOffset(Target target);
}
