/** The class for the symbol table for the body of a method or of a
 * nested block statement. */

package SymbolTable;

import AST.*;
import Helpers.*;
import java.util.*;

public class CodeSymbolTable extends NestedSymbolTable {
    // Instance variables

    /** the interface of local & formal variables declared in this scope */
    protected Map/*<String,LocalVarInterface>*/ local_variables;


    // Constructor

    /** create the symbol table for a statement list, given the symbol
     * table for the lexically enclosing scope */
    public CodeSymbolTable(SymbolTable enclosing_symbol_table) {
	super(enclosing_symbol_table);
	local_variables = new HashMap();
    }


    // Printing

    /** Return a printable string representation of the symbol table
     * information */
    public String toString(String indent) {
	String s = "CodeSymbolTable{\n";
	String nested_indent = ASTNode.nested_indent(indent);
	Iterator iter = local_variables.entrySet().iterator();
	while (iter.hasNext()) {
	    Map.Entry entry = (Map.Entry)iter.next();
	    String var_name = (String)entry.getKey();
	    LocalVarInterface var_interface = 
		(LocalVarInterface)entry.getValue();
	    s += indent + var_name + ": " + var_interface.toString() + "\n";
	}
	s += indent + "}";
	return s;
    }


    // Typechecking operations

    /** declare a local variable, checking for duplicates */
    public void declareLocalVariable(ResolvedType type, String name, 
				     int line_number)
		throws TypecheckCompilerException {
	if (local_variables.containsKey(name)) {
	    // we've already declared a local variable with this same name
	    throw new TypecheckCompilerException(
		    "redeclaring local variable named \"" + name + "\"", 
		    line_number);
	} else {
	    // record this new local variable
	    local_variables.put(name, new LocalVarInterface(name, type));
	}
    }

    /** Return the interface for the variable named 'name' declared in
     * this or some enclosing scope, or throw a TypecheckCompilerException if
     * the variable isn't declared. */
    public VarInterface lookupVariable(String name, int line_number)
		throws TypecheckCompilerException {
	Object vi = local_variables.get(name);
	if (vi == null) {
	    // variable not declared in this scope; search the
	    // lexically enclosing scope
	    return enclosing_symbol_table.lookupVariable(name, line_number);
	} else {
	    // found it; return its type
	    return (LocalVarInterface)vi;
	}
    }
}
