/** The class for (resolved) boolean types.  Only a single instance
 * should ever be created. */

package SymbolTable;

import Environment.*;
import IL.*;
import Helpers.*;

public class ResolvedBooleanType extends ResolvedType {
    // protected static variable, caching the unique representative of
    // this type
    protected static ResolvedBooleanType booleanType = 
	new ResolvedBooleanType();

    // Constructor (protected, so clients can't call it directly)
    protected ResolvedBooleanType() { }

    // public static operation, which is the main one clients should
    // use to get the unique instance of this type
    public static ResolvedBooleanType booleanType() { return booleanType; }


    // Printing operations
    public String toString() { 
	return "boolean"; 
    }


    // Testers

    /** return whether this type is a boolean type */
    public boolean isBooleanType() {
	return true;
    }


    // Evaluation operations

    /** Return the default initial value for a variable of this type */
    public Value defaultValue() {
	// false is the default value
	return new BooleanValue(false);
    }


    // Lowering operations

    /** Return the lowered version of this type */
    public ILType lower() {
	// booleans are represented by ints in the lowered language
	return ILType.intILType();
    }

    /** Return an IL expression representing the default value of this
     * type */
    public ILExpr defaultILExpr() {
	// false is the default value, which is represented by the
	// integer 0
	return new ILIntConstantExpr(0);
    }

    /** Return an IL value for initializing a global declaration
     * representing the default value of this type */
    public ILValue defaultILValue() {
	// false is the default value, which is represented by the
	// integer 0
	return new ILIntValue(0);
    }
}
