/** The class for (resolved) int types.  Only a single instance should
 * ever be created. */

package SymbolTable;

import Environment.*;
import IL.*;
import Helpers.*;

public class ResolvedIntType extends ResolvedType {
    // protected static variable, caching the unique representative of
    // this type
    protected static ResolvedIntType intType = new ResolvedIntType();

    // Constructor (protected, so clients can't call it directly)
    protected ResolvedIntType() { }

    // public static operation, which is the main one clients should
    // use to get the unique instance of this type
    public static ResolvedIntType intType() { return intType; }


    // Printing operations
    public String toString() { 
	return "int"; 
    }


    // Testers

    /** return whether this type is an int type */
    public boolean isIntType() {
	return true;
    }


    // Evaluation operations

    /** Return the default initial value for a variable of this type */
    public Value defaultValue() {
	// 0 is the default value
	return new IntValue(0);
    }


    // Lowering operations

    /** Return the lowered version of this type */
    public ILType lower() {
	return ILType.intILType();
    }

    /** Return an IL expression representing the default value of this
     * type */
    public ILExpr defaultILExpr() {
	// 0 is the default value
	return new ILIntConstantExpr(0);
    }

    /** Return an IL value for initializing a global declaration
     * representing the default value of this type */
    public ILValue defaultILValue() {
	// 0 is the default value
	return new ILIntValue(0);
    }
}
