/** The abstract class for declaration information about variables. */

package SymbolTable;

import Environment.*;
import IL.*;
import Helpers.*;

public abstract class VarInterface {
    // Instance variables
    protected String name;
    protected ResolvedType type;


    // Constructor
    public VarInterface(String n, ResolvedType t) {
	name = n;
	type = t;
    }


    // Printing
    public String toString() {
	return type.toString() + " " + name;
    }


    // Accessor
    public ResolvedType getType() {
	return type;
    }


    // Evaluation operations

    /** Return the current contents of this variable, reporting an
     * error if accessing the variable before it's been initialized */
    public abstract Value lookupVariable(CodeEnvironment env, int line_number) 
	throws EvalCompilerException;

    /** Update the current contents of this variable */
    public abstract void assignVariable(CodeEnvironment env, Value value, 
					int line_number);


    // Lowering (i.e., intermediate-language generation) operations

    /** generate intermediate-language code to assign to this
     * variable */
    public abstract void generateAssignment(ILExpr rhs_expr, 
					    ILFunDecl il_fun_decl);

    /** generate intermediate-language code to read from this
     * variable, returning an IL variable holding the result */
    public abstract ILExpr generateRead(ILFunDecl il_fun_decl);
}
